/*
 * Copyright (c) 2009, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#include <nios2.h>
#include <irq.h>

/* enable processor (ext.) interrupts */
void ei_set_pie(void)
{
   nios2_wrctl(status, NIOS2_CTL_STATUS_PIE);
}

/* disable processor (ext.) interrupts */
void ei_clr_pie(void)
{
   nios2_wrctl(status, 0x00);
}

/* single interrupt enable */
void ei_en(unsigned int irq)
{
   unsigned int irqn;

   if ((irq < 0) || (irq > 31))
      return;

   irqn = (unsigned int)0x01;
   irqn <<= irq;

   nios2_wrctl(ienable, nios2_rdctl(ienable) | irqn);
}

/* single interrupt enable */
void ei_di(unsigned int irq)
{
   unsigned int irqn;

   if ((irq < 0) || (irq > 31))
      return;

   irqn = (unsigned int)0x01;
   irqn <<= irq;

   nios2_wrctl(ienable, nios2_rdctl(ienable) & ~irqn);
}

/* single interrupt value read */
int ei_rd(unsigned int irq)
{
   unsigned int irqn;

   if ((irq < 0) || (irq > 31))
      return -1;

   irqn = (unsigned int)0x01;
   irqn <<= (irq-1);

   if((unsigned int)nios2_rdctl(ienable) & irqn)
      return 1;
   else
      return 0;
}

void ei_hdlr(void)
{
   unsigned int irqn = 0;
   unsigned int iactive;

   /* consider only irqs that are both enabled and pending */

   iactive = nios2_rdctl(ienable) & nios2_rdctl(ipending);

   /* bit0 is assumed to have the highest priority with this operation */

   while (iactive) {
      if (iactive & 1) {
        irq_entry(irqn);
      }
      iactive >>= 1;
      irqn++;
   }
}

