/*
 * Copyright (c) 2009, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#include <asm/mmap.h>
#include <serial.h>
#include <channel.h>
#include <sysutil.h>
#include <vt100.h>

#include <fb.h>
#include <fbcon.h>

#include <malloc.h>

#include <nios2.h>

#include <ei.h>
#include <irq.h>

#include <gfx2d/draw.h>
#include <gfx2d/sprite.h>
#include <gfx2d/tga.h>

#include "../data/rle/sprite_00.h"

#include <stdarg.h>

#include <math/math.h>

struct epcs_s{
   unsigned int sector_cnt;
   unsigned int sector_size;             
   unsigned int page_size;
};

unsigned char malloc_area[2*1024*1024] __attribute__ ((aligned(4)));;

unsigned short fb_0[480*272] __attribute__ ((aligned(0x100)));;
unsigned short fb_1[480*272] __attribute__ ((aligned(0x100)));;

/* timer */

static volatile unsigned int timeout = 0;

void timer_irq_hdlr(void)
{
   timeout = 1;
   alt_timer_irqclr(IO_TIMER_00_BASE);
}

/* ... */

void tiny(void)
{
   fb_t fb_16_00;
   sprite_t sprite_16_00;
   tga_info_t tga_info_00;

/*
   channel_register(STDIN, (channel_handler_t*)uart_getc, \
               (void*)IO_DFLT_UART | 0x80000000);
   channel_register(STDOUT, (channel_handler_t*)uart_putc, \
               (void*)(IO_DFLT_UART | 0x80000000));
   channel_register(STDERR, (channel_handler_t*)uart_putc, \
               (void*)(IO_DFLT_UART | 0x80000000));
*/
   irq_init();

   fb_init(fb_0);
   fbc_init((unsigned char*)fb_0, 480, 272);

   channel_register(STDOUT, (channel_handler_t*)fbc_ch_putchar_16, \
               (void*)(0x00));
   channel_register(STDERR, (channel_handler_t*)fbc_ch_putchar_16, \
               (void*)(0x00));

   alt_timer_init(IO_TIMER_00_BASE | 0x80000000, SYS_CLK_FREQ, 500);
   alt_timer_start(IO_TIMER_00_BASE | 0x80000000);

   irq_register(IO_TIMER_00_IRQ, (irq_handler_t*)timer_irq_hdlr, 0x00);

   /* ... */

   int i;
   unsigned short pixel_16;

   pixel_16 = rgb24_to_16(0xff, 0x5d, 0x0d);

   for(i=0; i<480*272; i++){
      fb_0[i] = pixel_16;
      fb_1[i] = pixel_16;
   }

   /* ... */

   printf("tiny nios2 [experimental_000]\n\n");

   malloc_init(malloc_area, sizeof(malloc_area), 128);
   malloc_stat();

   /* ... */

   irq_enable();

   fb_dma0_en();
   fb_l0_en();

   /* ... */

   fb_16_00.w = 480;
   fb_16_00.h = 272;
   fb_16_00.bpp = 16;
   fb_16_00.base = (unsigned char*)fb_0;

   tga_info(sprite_00, &tga_info_00);
   sprite_16_00.base = (unsigned char*)malloc(tga_info_00.w * tga_info_00.h * 2);
   tga24_to_sprite16(&tga_info_00, &sprite_16_00);

   malloc_stat();

   /* ... */

   area_t area_16_00;

   area_16_00.x = 0;
   area_16_00.y = 0;
   area_16_00.w = 48;
   area_16_00.h = 48;

   // sprite_area_blit_16(&fb_16_00, &sprite_16_00, &area_16_00, 380, 20);

   sprite_blit_16(&fb_16_00, &sprite_16_00, 380, 20);
   sprite_blit_alpha_16(&fb_16_00, &sprite_16_00, 356, 44, 0x80);

   float f;
   unsigned int* x;

   f = (float)(3.14159265);
/*
   printf("%f\n", f);

   printf("%d\n", fct(5));

   printf("%f\n", pow(0, 10));
   printf("%f\n", pow(1, 0));
   printf("%f\n", pow(2.125, 10));
*/
   printf("\n");

   printf("%.2f[rad] = %.2f[deg]\n", 1.0, deg(1.0));
   printf("cos(%.2f)[deg] = %.2f\n", 27.0, cos(rad(27.0)));
   printf("sin(%.2f)[deg] = %.2f\n", 27.0, sin(rad(27.0)));

   for(;;);
}

