/*
 * Copyright (c) 2010, Noel Lemouel
 * All rights reserved.
 */

#include <asm/mmap.h>
#include <string.h>
#include "fb.h"

static lcd_ctrl_t* gfx;

void fb_vsync(void)
{
   gfx->ctrl |= LCD_IEN;   /* enable interrupt */
   gfx->status = LCD_IVS;  /* clear interrupt */

   while(!(gfx->status & LCD_IVS));

   gfx->ctrl &= ~LCD_IEN;  /* disable interrupt */
}

void fb_dma0_en(void)
{
   gfx->ctrl |= (unsigned int)LCD_DMA0;   /* dma0 enable */

   while( ((gfx->status) & LCD_DMA0) != ((gfx->ctrl) & LCD_DMA0) );
}

void fb_dma0_di(void)
{
   gfx->ctrl &= ~LCD_DMA0;   /* l0 disable */

   while( ((gfx->status) & LCD_DMA0) != ((gfx->ctrl) & LCD_DMA0) );
}

void fb_l0_en(void)
{
   gfx->ctrl |= (unsigned int)LCD_L0;   /* l0 enable */

   while( ((gfx->status) & LCD_L0) != ((gfx->ctrl) & LCD_L0) );
}

void fb_l0_di(void)
{
   gfx->ctrl &= ~LCD_L0;   /* l0 disable */

   while( ((gfx->status) & LCD_L0) != ((gfx->ctrl) & LCD_L0) );
}

void fb_l0_remap_buffer(unsigned short* fb_base)
{
/*
   fb_dma0_di();
   fb_l0_di();
*/
   gfx->l0_dma_start = (unsigned int)fb_base;
/*
   fb_dma0_en();
   fb_l0_en();
*/
}

int fb_init(unsigned short* fb_base)
{

   gfx = (lcd_ctrl_t*)(IO_MM_GFX_BASE | 0x08000000);

   /* warm reset or sw download => ip already running */

   gfx->ctrl &= ~LCD_IEN;  /* disable interrupt */
   gfx->status = LCD_IVS;  /* clear interrupt */
   fb_dma0_di();
   fb_l0_di();

   /* ... */

   gfx->l0_dma_start = (unsigned int)fb_base;
   gfx->l0_dma_len = (unsigned int)(IO_MM_GFX_HRES*IO_MM_GFX_VRES);

   /* memset(fb_base, 0x00, IO_MM_GFX_HRES*IO_MM_GFX_VRES*2); */

   return 0;
}

