/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved.
 */

#include <asm_nios2/ei.h>
#include <sysutil.h>

#include "irq.h"

/* hardware interrupts */

struct irq_dsc {
   irq_handler_t *handler;
   void *arg;
   long count;
};

static struct irq_dsc irq[32];

static void irq_def_hdlr(void* arg)
{
   ei_di((unsigned int)arg);
   printf("irq: [critical] disabled unhandled irq #%d\n", (int)arg);
}

void irq_init(void)
{
   int i;

   /* assign the default handler to each one */
   for (i = 0; i < 32; i++) {
      irq[i].handler = (irq_handler_t *)irq_def_hdlr;
      irq[i].arg = (void *)i;
      irq[i].count = 0;
   }
}

int irq_register(int irqn, irq_handler_t *hdlr, void *arg)
{
   if ((irqn < 0) || (irqn > 31))
      return -1;

   if (hdlr) {
      irq[irqn].handler = hdlr;
      irq[irqn].arg = arg;
      ei_en(irqn); /* enable */
   } else {
      irq[irqn].handler = (irq_handler_t *)irq_def_hdlr;
      irq[irqn].arg = (void *)irq;
      ei_en(irqn); /* disable */
   }

   return 0;
}

void irq_entry(unsigned int irqn)
{
   irq[irqn].handler(irq[irqn].arg);
   irq[irqn].count++;
}

void irq_enable(void)
{
   ei_set_pie();
}

void irq_disable(void)
{
   ei_clr_pie();
}

void irq_info(void)
{
   printf ("\nactive irq information:\n\n");
   printf ("#   routine     arg         count\n");
   printf ("---------------------------------\n");

   /* ... */
}

