/*
 * Copyright (c) 2007, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

/* ultra lite flash driver for s29glxxN flash series */
/* word mode only */

/* tested working with S29GL128N / S29GL256N / AM29LV640MH */

#include "s29gl.h"

#define DEBUG

// #ifdef DEBUG
#include <sysutil.h>
// #endif

static s29gl_flash_profile_t flash_profile;
static s29gl_flash_cmd_t flash_cmd;

static s29gl_flash_cmd_t cmd;

s32 s29gl_init(u32 flash_base)
{
    u16 cfi_ascii_QRY[3];
    u16 erase_block_region1_info[4];
    u32 i;

    flash_profile.base_address = flash_base;
    
    cmd.addr0 = (volatile u16 *) (flash_profile.base_address + (0x555<<1));
    cmd.addr1 = (volatile u16 *) (flash_profile.base_address + (0xAAA<<1));
    cmd.device_ID_x01 = (volatile u16 *) (flash_profile.base_address + (0x001<<1));
    cmd.device_ID_x0e = (volatile u16 *) (flash_profile.base_address + (0x001<<1));    
    cmd.device_ID_x0f = (volatile u16 *) (flash_profile.base_address + (0x001<<1));
    cmd.manufacturer_ID = (volatile u16 *) (flash_profile.base_address + (0x000<<1));

    /* device reset */
    *cmd.addr0 = (u16)0xf0;
    
    /* CFI query mode */
    cmd.cfi_addr = (volatile u16 *) (flash_profile.base_address + (0x55<<1));
    *cmd.cfi_addr = (u16)0x98;
    
    /* read the 'QRY' string */

    /* cfi_addr = flash_profile.base_address + (0x10<<flash_profile.nbyte_mode); */
    cmd.cfi_addr = (volatile u16 *)  (flash_profile.base_address + 0x20);
    for(i=0; i<3; i++){
        cfi_ascii_QRY[i] = *cmd.cfi_addr;
        cmd.cfi_addr++;
    }
    
    if(!((cfi_ascii_QRY[0] == (u16)0x0051) &&
         (cfi_ascii_QRY[1] == (u16)0x0052) &&
         (cfi_ascii_QRY[2] == (u16)0x0059))) {
#ifdef DEBUG
      printf("%s: fatal: no CFI flash found at 0x%08x\n", "s29gl", flash_profile.base_address);
#endif    
    return -1; }
    
    /* now get the mem size */
    
    /* cfi_addr = flash_profile.base_address + (0x27<<flash_profile.nbyte_mode); */
    cmd.cfi_addr = (volatile u16 *) (flash_profile.base_address + 0x4e);
    switch (*cmd.cfi_addr)
    {
        case(0x17):
            flash_profile.device_size = 64;
            break;
        case(0x18):
            flash_profile.device_size = 128;
            break;
        case(0x19):
            flash_profile.device_size = 256;
            break;
        case(0x1A):
            flash_profile.device_size = 512;
            break;
        default:
#ifdef DEBUG
               // printf("%s: fatal: error reading CFI table\n", "s29gl");
#endif
            return -1;
    }
#ifdef DEBUG
      printf("%s: found %dMB CFI device at 0x%08x\n",  "s29gl", (flash_profile.device_size)>>3, flash_profile.base_address);
#endif

    /* device geometry definition */

    /* cfi_addr = flash_profile.base_address + (0x2c<<flash_profile.nbyte_mode); */
    cmd.cfi_addr = (volatile u16 *) (flash_profile.base_address + 0x58);
    if((*cmd.cfi_addr) != (u16)0x01){
#ifdef DEBUG
         // printf("%s: fatal: non-uniform device not supported\n", "s29gl");
#endif   
      // return ERR_FATAL;
    }

    /* cfi_addr = flash_profile.base_address + (0x2d<<flash_profile.nbyte_mode); */
    cmd.cfi_addr = (volatile u16 *) (flash_profile.base_address + 0x5a);     
    for(i=0; i<4; i++){
      erase_block_region1_info[i] = *cmd.cfi_addr;
      cmd.cfi_addr++;
    }

    flash_profile.sector_count = (erase_block_region1_info[0] | (erase_block_region1_info[1] << 8)) + 1;
    flash_profile.sector_size = ((flash_profile.device_size) << 7) / flash_profile.sector_count;
#ifdef DEBUG
       // printf("%s: geometry: %d sectors of %dkB\n", "s29gl", flash_profile.sector_count, flash_profile.sector_size);
#endif
    /* quit CFI query mode (reset) */
    *cmd.addr0 = (u16)0xf0;

    /* read manufacturer ID */

    *cmd.addr0 = (u16)0xaa;
    *cmd.addr1 = (u16)0x55;
    *cmd.addr0 = (u16)0x90;
    
#ifdef DEBUG
      // printf("%s: manufacturer ID: 0x%04x\n", "s29gl", *cmd.manufacturer_ID);
#endif

    /* reset */
    *cmd.addr0 = (u16)0xf0;

    /* read device ID */

    *cmd.addr0 = (u16)0xaa;
    *cmd.addr1 = (u16)0x55;
    *cmd.addr0 = (u16)0x90;

#ifdef DEBUG
       // printf("%s: device ID: 0x%04x\n", "s29gl", *cmd.device_ID_x01);
#endif
 
    /* reset */
    *cmd.addr0 = (u16)0xf0;

    return 0;
}

s32 s29gl_erase_all(void)
{
    u16 read;
    bool_t finished;

    /* should check if the flash is already empty */

    *cmd.addr0 = (u16)0xf0; /* reset */

    *cmd.addr0 = (u16)0xAA; /* erase setup */
    *cmd.addr1 = (u16)0x55;
    *cmd.addr0 = (u16)0x80;
    *cmd.addr0 = (u16)0xAA; /* chip erase */
    *cmd.addr1 = (u16)0x55;
    *cmd.addr0 = (u16)0x10;

    *cmd.addr0 = flash_profile.base_address;

    /* disable interrupts... */
    
    finished = FALSE;
    while (finished == FALSE)
    {
        read = *cmd.addr0;
        if ((read & 0x0080) != 0)
        {
            finished = TRUE;
        }
        else if ((read & 0x0020) != 0)
        {
            read = *cmd.addr0;
            if ((read & 0x0080) != 0)
            {
                finished = TRUE;
            }
            else
            {
                return -1;
            }
        }
    }

    /* enable interrupts... */
    
    *cmd.addr0 = (u16)0xf0; /* reset */

    return 0;
}

s32 s29gl_sector_erase(u32 sector_base)
{
    bool_t finished;

    u16 read;
/*
    if(sector_num > flash_profile.sector_count)
        return -ERR_BAD_PARAMETER;
*/
   cmd.sector_base = (volatile u16*)sector_base;

/*
    cmd.sector_base = (volatile u16*) \
                      (flash_profile.base_address + (sector_num * (flash_profile.sector_size<<10)));

    if(s29gl_is_sector_empty(sector_num)) {
    #ifdef S29GL_VERBOSE
       printf("%s: sector_erase: erase not required for sector %d [base 0x%08x]\n", \
             "s29gl", sector_num, cmd.sector_base);
    #endif
       return OK;
    }
*/    
    *cmd.addr0 = (u16)0xf0;   /* reset */

    *cmd.addr0 = (u16)0xaa;
    *cmd.addr1 = (u16)0x55;
    *cmd.addr0 = (u16)0x80;
    *cmd.addr0 = (u16)0xaa;    
    *cmd.addr1 = (u16)0x55;
    
    *cmd.sector_base = (u16)0x30;

    /* disable interrupts... */
    
    finished = FALSE;
    while (finished == FALSE)
    {
        read = *cmd.sector_base;
        if ((read & 0x0080) != 0)
        {
            finished = TRUE;
        }
        else if ((read & 0x0020) != 0)
        {
            read = *cmd.sector_base;
            if ((read & 0x0080) != 0)
            {
                finished = TRUE;
            }
            else
            {
#ifdef DEBUG
                  printf("%s: sector_erase: fatal: [base: 0x%08x] not erased\n", \
                        "s29gl", cmd.sector_base);
#endif
                return -1;
            }
        }
     }

    /* enable interrupts... */
    
    *cmd.addr0 = (u16)0xf0;   /* reset */

#ifdef DEBUG
      // printf("%s: sector_erase: [base: 0x%08x] erased\n", "s29gl", cmd.sector_base);
#endif

    return 0;
}

s32 s29gl_byte_write(u32 flash_base, u32 word_address, u16 data)
{
   u32 finished;
   u16 read;

   // *(unsigned short*)(flash_base) = (u16)0xf0;   /* reset */

   *(u16*)(flash_base + (0x555<<1)) = (u16)0xaa;
   *(u16*)(flash_base + (0xaaa<<1)) = (u16)0x55;
   *(u16*)(flash_base + (0x555<<1)) = (u16)0xa0;
   *(u16*)(word_address) = data;

   finished = FALSE;
   while (finished == FALSE)
   {
      read = *(u16*)(word_address);
      if ((read & 0x0080) == (data & 0x0080))
      {
         finished = TRUE;
      }
      else if ((read & 0x0020) == 0x0020)
      {
         read = *(u16*)(word_address);
         if ((read & 0x0080) == (data & 0x0080))
         {
            finished = TRUE;
         }
         else
         {
            printf("%s: programming: fatal: write failed at 0x%08x [value: 0x%08x]", \
                           "s29gl", word_address, data);
            return -1;
         }
      }
   }

   return 0;
}

s32 s29gl_buffer_write(u32 flash_base, u32 offset, u16* buf, u32 len)
{
   u32 i, finished;
   u16* word_address;
   u16 read;

   *(u16*)(flash_base) = (u16)0xf0;   /* reset */

   word_address = (u16*)(flash_base + offset);

   for(i=0; i<len; i++)
   {

      *(u16*)(flash_base + (0x555<<1)) = (u16)0xaa;
      *(u16*)(flash_base + (0xaaa<<1)) = (u16)0x55;
      *(u16*)(flash_base + (0x555<<1)) = (u16)0xa0;
      *word_address = buf[i];

      finished = FALSE;
      while (finished == FALSE)
      {
         read = *word_address;
         if ((read & 0x0080) == (buf[i] & 0x0080))
         {
            finished = TRUE;
         }
         else if ((read & 0x0020) == 0x0020)
         {
            read = *word_address;
            if ((read & 0x0080) == (buf[i] & 0x0080))
            {
               finished = TRUE;
            }
            else
            {
               printf("%s: programming: fatal: write failed at 0x%08x [value: 0x%04x]\n", \
                           "s29gl", word_address, buf[i]);
               return -1;
            }
         }
      }
      word_address++;
   }

   return 0;
}

void s29gl_reset(u32 flash_base)
{
   *(u16*)(flash_base + (0x555<<1)) = (u16)0xaa;
   *(u16*)(flash_base + (0xaaa<<1)) = (u16)0x55;
   *(u16*)(flash_base + (0x555<<1)) = (u16)0xf0;
}

