/*
 * Copyright (c) 2009, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

/* 16bits linear framebuffers virtual console */

#include <gfx2d/draw.h>
#include "font_8x8.c"
#include "fbcon.h"

#define FBCON_CURSOR_ACTIVE
#define FBCON_CURSOR_BLINK

#ifdef FBCON_CURSOR_BLINK
#include <asm/mmap.h>
#include <irq.h>
#include <timer.h>
#endif

/* 8x8 font */
#define font_w 8
#define font_h 8

static fb_t vcon_fb;

volatile static struct vcon_s vcon;

#ifdef FBCON_CURSOR_BLINK
static void toggle_cursor(void)
{
   alt_timer_irqclr(IO_TIMER_01_BASE | 0x80000000);

   if(vcon.cursor_lock) return;

   if(vcon.cursor_en)
   {
      fbc_charblt_16(0x00, vcon.cursor_x, vcon.cursor_y);
      vcon.cursor_en = 0;
   }else{
      fbc_charblt_16(0xdb, vcon.cursor_x, vcon.cursor_y);
      vcon.cursor_en = 1;
   }
}
#endif

int fbc_charblt_16(int c, int col, int row)
{
   unsigned int i, j, k, l;
   volatile unsigned short *fb;  /* d-cache bypass w/ -mno-cache-volatile */

   fb = (unsigned short*)vcon_fb.base;

   k = c << 3;
   l = (col * font_w) + (row * font_h * vcon_fb.w);

   for(j=0; j<8; j++)
   {
      c = fontdata_8x8[k];
      for(i=0; i<8; i++)
      {
         if(c & 0x80)
            fb[i+l] = 0xffff;
         else
            fb[i+l] = 0x0000;
         c <<= 1;
      }
      k++;
      l+=vcon_fb.w;
   }

   /* flush_dcache(); */

   return 0;
}

void fbc_hscroll_16(void)
{
   unsigned int *ps, *pe;
   unsigned long count;

   ps = (unsigned int *)vcon_fb.base; 
   pe = ps + ((vcon_fb.w << 3) / 2);
   count = (vcon_fb.w * (vcon_fb.h - font_h)) / 2;
   
   while(count--) *ps++ = *pe++;

   /* blank last line */

   ps = (unsigned int *)((unsigned int)vcon_fb.base + ( ((vcon_fb.w << 5) * 33) / 2));

   count = (vcon_fb.w << 2); /* mmapfb.width * 4 */

   while(count--) *ps++ = 0x00000000; 

   /* flush_dcache(); */
}

/* channel interface wrapper */
int fbc_ch_putchar_16(int base, char c)
{
   return fbc_putchar_16((int)c);
}

int fbc_putchar_16(int c)
{
   /* special char / TODO replace by real term emulation */

   vcon.cursor_lock = 1;
   
   switch(c){
      case '\r':
         vcon.cursor_x = 0;
#ifdef FBCON_CURSOR_ACTIVE
         fbc_charblt_16(0x00, vcon.cursor_x, vcon.cursor_y);   /* cursor */
#endif
         break;
      case '\n':
#ifdef FBCON_CURSOR_ACTIVE
         fbc_charblt_16(0x00, vcon.cursor_x, vcon.cursor_y);   /* cursor */
#endif
         vcon.cursor_x = 0;
         vcon.cursor_y++;
         break;
      default:
         fbc_charblt_16(c, vcon.cursor_x++, vcon.cursor_y);
         break;
   }

   /* check for borders */

   if (vcon.cursor_x == vcon.x_max_chr) {
      vcon.cursor_x = 0;
      vcon.cursor_y++;      
   }

   if (vcon.cursor_y == vcon.y_max_chr) {
      fbc_hscroll_16();
      vcon.cursor_y--;
   }
#ifdef FBCON_CURSOR_ACTIVE
   fbc_charblt_16(0xdb, vcon.cursor_x, vcon.cursor_y);   /* cursor */
#endif
   vcon.cursor_lock = 0;

   return (int)c;
}

int fbc_puts_16(char* str)
{
	int i=0;

	while(str[i])
	{
      fbc_putchar_16(str[i]);
      i++;
	}
   fbc_putchar_16('\n');

	return 0;
}

int fbc_init(unsigned char* base, unsigned short w, unsigned short h)
{
   /* check if width/height are ok ? */

   vcon_fb.w = w;
   vcon_fb.h = h;
   vcon_fb.base = base;

   vcon.x_max_chr = w / font_w;
   vcon.y_max_chr = h / font_h;

   vcon.cursor_x = 0;
   vcon.cursor_y = 0;
   vcon.cursor_en = 0;
   vcon.cursor_lock = 0;

   /* cursor blink */

#ifdef FBCON_CURSOR_BLINK
   alt_timer_init(IO_TIMER_01_BASE | 0x80000000, SYS_CLK_FREQ, 250);
   irq_register(IO_TIMER_01_IRQ, (irq_handler_t*)toggle_cursor, 0x00);
   alt_timer_start(IO_TIMER_01_BASE | 0x80000000);
#endif

   return 0;
}

