/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

/* non-reentrant */

#include <sysutil.h>
#include <stddef.h>
#include <string.h>

/* static int minit = 0; */

struct mstat_s{
	unsigned int mmem_base;
	unsigned int mmem_size;
	unsigned int mblk_size;
	/* unsigned int mtoc_base; */
	unsigned int mtoc_size;
	unsigned int mdata_base;
	unsigned int mdata_size;
	unsigned int mdata_blk_cnt;
	unsigned int mdata_lfr;
};

struct mtoc_entry_s{
	unsigned int addr;
	unsigned int id;
};

static struct mstat_s mstat;

void malloc_stat(void)
{
   int i;
   unsigned int eblk;
   struct mtoc_entry_s* t_entry;

#ifdef DEBUG
	printf("--------- mstat ---------\n");
	printf("mmem_base    : 0x%08x\n", mstat.mmem_base);
	printf("mmem_size    : 0x%08x\n", mstat.mmem_size);
	printf("mblk_size    : 0x%08x\n", mstat.mblk_size);
	printf("mtoc_base    : 0x%08x\n", mstat.mmem_base);
	printf("mtoc_size    : 0x%08x\n", mstat.mtoc_size);
	printf("mdata_base   : 0x%08x\n", mstat.mdata_base);
	printf("mdata_size   : 0x%08x\n", mstat.mdata_size);
	printf("mdata_blk_cnt: 0x%08x\n", mstat.mdata_blk_cnt);
	printf("mdata_lfr    : 0x%08x\n", mstat.mdata_lfr);
#endif

   t_entry = (struct mtoc_entry_s*)mstat.mmem_base;	/* = mtoc_start*/

   eblk = 0;
	for(i=0; i<mstat.mdata_blk_cnt; i++)
	{
		if(!(t_entry[i].addr)) eblk++;
	}

   printf("free: %0.2f%%\n", (float)((float)eblk*100/(float)mstat.mdata_blk_cnt));

   /* printf("free: %d%% [%d/%d]\n", eblk*100/mstat.mdata_blk_cnt, eblk, mstat.mdata_blk_cnt); */

}

int malloc_init(unsigned char* MMEM_BASE, size_t MMEM_SIZE, size_t MBLK_SIZE)
{
	unsigned int i;

   /* check if MMEM_BASE is 32bits aligned */
   /* check if MBLK_SIZE is a multiple of 32bits */

	mstat.mmem_base = (unsigned int)MMEM_BASE;
	mstat.mmem_size = (unsigned int)MMEM_SIZE;
	mstat.mblk_size = (unsigned int)MBLK_SIZE;

   i=0;
   while(((i*sizeof(struct mtoc_entry_s))+(i*MBLK_SIZE)) < MMEM_SIZE) i++; i--;
/*
   if((i*sizeof(struct mtoc_entry_s))%MBLK_SIZE) i--;

   printf("%d / 0x%08x / 0x%08x\n", i, i, i*sizeof(struct mtoc_entry_s));
   printf("0x%08x\n", (i*sizeof(struct mtoc_entry_s))%MBLK_SIZE); return -1;
*/
	mstat.mtoc_size = i*sizeof(struct mtoc_entry_s);
	mstat.mdata_base = (unsigned int)MMEM_BASE+mstat.mtoc_size;
   mstat.mdata_size = i*MBLK_SIZE;
	mstat.mdata_blk_cnt = i;
  	mstat.mdata_lfr = MMEM_SIZE-(i*(sizeof(struct mtoc_entry_s)+MBLK_SIZE));

	/* clear toc */
   memset((void *)MMEM_BASE, 0x00, mstat.mtoc_size);

	return 0;
}

void *malloc_id(size_t size, int id)
{
	int i, j, offset;
	struct mtoc_entry_s* t_entry;
	unsigned int rblk;
	unsigned int eblk;
	unsigned int maddr;

	if(!size) return NULL;

	/* if(size > mstat.mdata_size) return 0x00; */

   /* # of block(s) required */

	rblk = size / mstat.mblk_size;
/*	 printf("rblk: %d\n", rblk);	*/
	if((size % mstat.mblk_size) > 0) rblk++;

#ifdef DEBUG
	 printf("rblk: %d [0x%x]\n", rblk, size);
#endif

	t_entry = (struct mtoc_entry_s*)mstat.mmem_base;	/* = mtoc_start*/

	/* look for a free spot (rblk empty block(s) in a row) */

	eblk = 0;
	for(i=0; i<mstat.mdata_blk_cnt; i++)
	{
		(!(t_entry[i].addr)) ? (eblk++) : (eblk=0);
		if(eblk == rblk) break;
	}
	if(eblk != rblk) return NULL;

#ifdef DEBUG
	 printf("eblk: %d / i: %d\n", eblk, i);
#endif

/*
	 printf("t_entry: 0x%08x\n", &(t_entry[8].addr));
	 printf("t_entry: 0x%08x\n", &(t_entry[8].id));
*/
	/* calculate start address */

	i++;
	offset = i-rblk;
	maddr = mstat.mdata_base + (offset*mstat.mblk_size);

#ifdef DEBUG
	 printf("maddr: 0x%08x\n", maddr);
#endif

	/* initialize toc */

	for(j=(i-rblk); j<i; j++)
	{
		t_entry[j].addr = maddr;
		t_entry[j].id = maddr;
	}	

	return (void *)maddr;
}

void *malloc(size_t size)
{
	return (void *)malloc_id(size, 0x00);
}

void free(void *ptr)
{
	int i;
	struct mtoc_entry_s* t_entry;

	t_entry = (struct mtoc_entry_s*)mstat.mmem_base;	/* = mtoc_start*/

	for(i=0; i<mstat.mdata_blk_cnt; i++)
	{
		if((t_entry[i].addr)==(unsigned int)ptr)
		{
			t_entry[i].addr = 0x00;
			t_entry[i].id = 0x00;
		}
	}
}

void *realloc(void *ptr, size_t size)
{
	int i;
   int blk = 0;
	struct mtoc_entry_s* t_entry;

	t_entry = (struct mtoc_entry_s*)mstat.mmem_base;	/* = mtoc_start*/

	for(i=0; i<mstat.mdata_blk_cnt; i++)
	{
		if((t_entry[i].addr)==(unsigned int)ptr) blk++;
	}

   if((blk*mstat.mblk_size) <= size){
      return ptr;
   }else{
      free(ptr);
      return malloc(size);
   }
}

void *calloc(size_t nelem, size_t elsize)
{
   void *ptr;

   ptr = malloc(nelem*elsize);
   if(ptr) memset((void *)ptr, 0x00, nelem*elsize);

   return ptr;
}

